﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;


namespace 太陽電池モデル計算
{

    public partial class Form1 : Form
    {
        double m_V;
        double m_IL;
        double m_Io;
        double m_q;
        double m_k;
        double m_T;
        double m_Rs;
        double m_Rsh;
        double m_n;
        double m_dV;
        double m_II;
        double m_VV;

        System.IO.StreamWriter log;
        String logPath;
        string file;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // 計算開始
            read_par();
            open_log();

            while (m_VV <= m_V)
            {
                // 初期化
                m_II = m_IL;

                // 計算
                m_II = cal_solarcell(m_II, m_VV, m_T);
                if (m_II < 0) break;

                // ログ出力
                log.Write(m_VV.ToString() + ",");
                log.Write(m_II.ToString() + ",\n");

                // 電圧値更新
                m_VV += m_dV;
            }

            // ログそっとじ
            log.Close();

            // 表示
            MessageBox.Show("計算が終わりました",
            "エラー",
            MessageBoxButtons.OK,
            MessageBoxIcon.Exclamation);
        }

        private void read_par()
        {
            // テキストボックスから値を入力
            m_V     = double.Parse(textBox1.Text);
            m_IL    = double.Parse(textBox2.Text);
            m_Io    = double.Parse(textBox3.Text);
            m_q     = double.Parse(textBox4.Text);
            m_k     = double.Parse(textBox5.Text);
            m_T     = double.Parse(textBox6.Text);
            m_Rs    = double.Parse(textBox7.Text);
            m_Rsh   = double.Parse(textBox8.Text);
            m_dV    = double.Parse(textBox9.Text);

            m_n =  double.Parse(textBox10.Text);
            m_VV = 0;
        }

        private double cal_solarcell(double I,double V, double T)
        {
            double x0;
            double x1;
            double xx = 0;
            double a;
            double res;

            x0 = -2 * I;
            x1 = I;

            while (System.Math.Abs((x0 - x1) / (x0 + x1)) > 10E-6)
            {
                // 2分法計算
                // http://oshiete.goo.ne.jp/qa/5958862.html
                //---
                xx = (x0 + x1) / 2;
                a = m_q*(V + xx * m_Rs) / (m_n * m_k * T);

                // 計算
                res = I - ((V + xx * m_Rs) / m_Rsh) - xx - m_Io * (System.Math.Exp(a) - 1);

                // 判定
                if (res > 0)
                {
                    x0 = xx;
                }
                else
                {
                    x1 = xx;
                }
            }

            return xx;
        }

        private void open_log()
        {
            file = System.DateTime.Now.ToString("yyyyMMddHHmmss");
            file = file.Replace("/", "");
            file = file.Replace(":", "");
            file = file.Replace(" ", "");
            logPath = AppDomain.CurrentDomain.BaseDirectory + "Log_solar_" + file + ".csv";
            log = new System.IO.StreamWriter(logPath, true, System.Text.Encoding.GetEncoding("shift_jis"));

            log.Write("# V[V], I[A]\n");
        }
    }
}
